/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/*----------------------------------------------------------------------------------**
** Copyright (c) 2000, Peter Siebold                                                **
** All rights reserved.                                                             **
** Redistribution and use in source and binary forms, with or without modification, **
** are permitted provided that the following conditions are met:                    **
**                                                                                  **
**                                                                                  **
** Redistributions of source code must retain the above copyright notice, this list **
**  of conditions and the following disclaimer.                                     **
**                                                                                  **
** Redistributions in binary form must reproduce the above copyright notice, this   **
** list of conditions and the following disclaimer in the documentation and/or      **
** other materials provided with the distribution.                                  **
**                                                                                  **
** Neither the name of "aems555" nor the names of its contributors may be used      **
** to endorse or promote products derived from this software without specific prior **
** written permission.                                                              **
**                                                                                  **
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''    **
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE        **
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   **
** DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY      **
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES       **
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     **
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON   **
** ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          **
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS    **
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                     **
**----------------------------------------------------------------------------------**
**																					**
**----------------------------------------------------------------------------------**
**    EFI code for the ppc555														**
**																					**
**	File:		adc.c																**
**	Desc:		Support for the MPC555 QUEUED ANALOG-TO-DIGITAL CONVERTER			**
**																					**
**	Reference:	Motorola AN1791														**
**																					**
**	Author:		Peter Siebold														**
**	Tools:   	GNU tool chain														**
**																					**
**----------------------------------------------------------------------------------*/

#include "hw.h"
#include "adc.h"

QADCREG   *adcreg   = (QADCREG *)    QADCREGBASE;	//defined in hw.h

u_short skip;

#include "counts2mV.h"


/*---------------------------------------------------------------------------*/
/*  Initialize QADC for aquisition of 32 channels of 10 bit A/D at the lowest
	supported rate.  This assumes 4 8 channel external multiplexers

	called once at startup
*/

void ADCInit(void)
{
	unsigned int i;
	
	/* The QADC64 Module configuration register. */
	
	adcreg->mcr.b.STOP = 0; 	/* Enable the QADC64 */
	adcreg->mcr.b.FRZ = 0;		/* Ignore the freeze bit */
	adcreg->mcr.b.SUPV = 0; 	/* allow user access */
	
	/* The QADC64 Interrupt register */
	
	adcreg->intr.b.IRL1 = QADC_IRQL;
								/* interrupt request level. */

	
	/* The QADC64 Control register 0 
	
	adcreg->cr0.b.MUX = 1;		/* externally muxed */
	adcreg->cr0.b.TRG = 0;		/* trigger assignment */
	adcreg->cr0.b.PSH = 0x3;	/* high prescale of 4 */
	adcreg->cr0.b.PSL = 0x3;	/* low prescale of 4 */
	
	/* The QADC64 Control register 1
	
	adcreg->cr1.b.CIE1=1;		/* queue completion interrupts enabled */
	adcreg->cr1.b.PIE1=0;		/* queue 1 pause interrupts disabled */
	adcreg->cr1.b.SSE1=0;		/* single scan enable */
/*	adcreg->cr1.MQ1=0x14;		/* Queue 1 Operating Mode -- continuous scan */
								/* should wait until all is set up? */

	/* The QADC64 Control register 2
		
/* 	adcreg.b.cr2.MQ2=0x40;		/* disable Queue 2 -- default value */


	/* CCW
	number of channels is hard coded at 32.  need to change
	*/
	
	
	for(i=0;i<32;i++)
	{
		adcreg->ccw[i].w = 0xC0 | i;

	}
	
	adcreg->ccw[i].w	= 0xFF; /* conversion command word. End of Queue command */		
	

	adcreg->ccw[i].w	= 0xFC; /* conversion command word. Vrh */
	adcreg->ccw[i+1].w	= 0xFD; /* conversion command word. Vrl */
	adcreg->ccw[i+2].w	= 0xFE; /* conversion command word. (Vrh-Vrl)/2 */
	adcreg->ccw[i+3].w	= 0xFF; /* conversion command word. End of Queue command */		
	
	/*
	Now start the whole thing going.
	*/

	
	/* adcreg->cr1.b.MQ1 = 0x14;	/* Queue 1 Operating Mode -- continuous scan */
								/* fastest */

					
	adcreg->cr1.b.MQ1 = 0x1E;	/* Queue 1 Operating Mode -- continuous scan */
								/* slowest */

	skip = 0;

} /* End of ADCInit function */



/*---------------------------------------------------------------------------*/
/*  Initialize QADC for aquisition of 32 channels of 10 bit A/D at the lowest
	supported rate.  This assumes 4 8 channel external multiplexers

	called once at startup
*/

void ADCIntrHndlr(void)
{
	u_short status, result, i;
	char inttmpString[20];


	
	status = adcreg->sr0.w;
	
	if((status & 0x8000) == 0x8000)
	{
		/*
		que1 completion flag set

		First copy data into global memory location, this is currently a waste
		but I may want to implement a digital filter here for each channel.

		May not need this, but for now I'm going to use it

		*/

		for(i=0;i<32;i++)
		{
			addata[i] = adcreg->rjurr[i].w;
		}


		adcreg->sr0.b.CF1 = 0;
		/*adcreg->sr0.w = 0;


		skip++;

		This can be used during debugging to write the results of each conversion
		to the debug port directly

		if(skip > 9)
		{

			result = adcreg->rjurr[1].w;
			itoa(result, inttmpString);
			puts2(inttmpString);

			puts2(" ");

			result = count2mV[adcreg->rjurr[3].w];
			itoa(result, inttmpString);
			puts2(inttmpString);
			puts2("\n");

			skip = 0;

		}

		*/
	 
	}
	else
	{
		/*
		Should never get here, used during debugging only.  Need to implement a global
		error handler.
		*/

		htoa(status, inttmpString);
		puts2(inttmpString);
		puts2("\n");
		adcreg->sr0.w = 0;
	}


} /* End of ADCIntrHndlr function */
