/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/*----------------------------------------------------------------------------------**
** Copyright (c) 2000, Peter Siebold                                                **
** All rights reserved.                                                             **
** Redistribution and use in source and binary forms, with or without modification, **
** are permitted provided that the following conditions are met:                    **
**                                                                                  **
**                                                                                  **
** Redistributions of source code must retain the above copyright notice, this list **
**  of conditions and the following disclaimer.                                     **
**                                                                                  **
** Redistributions in binary form must reproduce the above copyright notice, this   **
** list of conditions and the following disclaimer in the documentation and/or      **
** other materials provided with the distribution.                                  **
**                                                                                  **
** Neither the name of "aems555" nor the names of its contributors may be used      **
** to endorse or promote products derived from this software without specific prior **
** written permission.                                                              **
**                                                                                  **
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''    **
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE        **
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   **
** DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY      **
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES       **
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     **
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON   **
** ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          **
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS    **
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                     **
**----------------------------------------------------------------------------------**
**																					**
**----------------------------------------------------------------------------------**
**    EFI code for the ppc555														**
**																					**
**	File:		adc.h																**
**	Desc:		Support for the MPC555 QUEUED ANALOG-TO-DIGITAL CONVERTER			**
**																					**
**	Reference:	Motorola AN1791														**
**																					**
**	Author:		Peter Siebold														**
**	Tools:   	GNU tool chain														**
**																					**
**----------------------------------------------------------------------------------*/

#ifndef		__adc555_H
#define		__adc555_H


typedef union
{
    struct
    {
	volatile u_short		STOP	:	1;
    	volatile u_short		FRZ	:	1;
    	u_short					:	6;
    	volatile u_short		SUPV	:	1;
	u_short					:	7;
    } b;
    short w;
} QADC64MCR_;

typedef union
{
    struct
    {
	volatile u_short		IRL1	:	5;
    	volatile u_short		IRL2	:	5;
    	u_short					:	6;
    } b;
    short w;
} QADC64INT_;


typedef union
{
    struct
    {
	volatile u_short		MUX	:	1;
    	u_short					:	2;
    	volatile u_short		TRG	:	1;
    	u_short					:	3;
    	volatile u_short		PSH	:	5;
    	volatile u_short		PSA	:	1;
    	volatile u_short		PSL	:	3;
    } b;
    short w;
} QACR0_;


typedef union
{
    struct
    {
	volatile u_short		CIE1	:	1;
	volatile u_short		PIE1	:	1;
	volatile u_short		SSE1	:	1;
	volatile u_short		MQ1	:	5;
    	u_short					:	8;
    } b;
    short w;
} QACR1_;

typedef union
{
    struct
    {
	volatile u_short		CIE2	:	1;
	volatile u_short		PIE2	:	1;
	volatile u_short		SSE2	:	1;
	volatile u_short		MQ2	:	5;
	volatile u_short		RESUME	:	1;
	volatile u_short		BQ2	:	7;
    } b;
    short w;
} QACR2_;

typedef union
{
    struct
    {
	volatile u_short		CF1	:	1;
	volatile u_short		PF1	:	1;
	volatile u_short		CF2	:	1;
	volatile u_short		PF2	:	1;
	volatile u_short		TOR1	:	1;
	volatile u_short		TOR2	:	1;
	volatile u_short		QS	:	4;
	volatile u_short		CWP	:	6;
    } b;
    short w;
} QASR0_;

typedef union
{
    struct
    {
	u_short					:	2;
	volatile u_short		CWPQ1	:	6;
	u_short					:	2;
	volatile u_short		CWPQ2	:	6;
    } b;
    short w;
} QASR1_;

typedef union
{
    struct
    {
	u_short					:	6;
	volatile u_short		P	:	1;
	volatile u_short		BYP	:	1;
	volatile u_short		IST	:	2;
	volatile u_short		CHAN	:	6;
    } b;
    short w;
} CCW_;

typedef union
{
    struct
    {
	u_short					:	6;
	volatile u_short		RESULT	:	10;

    } b;
    short w;
} RJURR_;

typedef union
{
    struct
    {
	volatile u_short		S	:	1;
	volatile u_short		RESULT	:	9;
	u_short					:	6;
    } b;
    short w;
} LJSRR_;

typedef union
{
    struct
    {
	volatile u_short		RESULT	:	10;
	u_short					:	6;
    } b;
    short w;
} LJURR_;

typedef struct QADC_ {
	volatile QADC64MCR_	mcr;
	volatile u_short	tcr;
	volatile QADC64INT_	intr;
	volatile u_short	portdata;
	
	volatile u_short	ddrqa;
	volatile QACR0_		cr0;
	volatile QACR1_		cr1;
	volatile QACR2_		cr2;
	volatile QASR0_		sr0;
	volatile QASR1_		sr1;
	volatile u_short	reserved[246];
	
	volatile CCW_		ccw[64];
	
	volatile RJURR_		rjurr[64];
	volatile LJSRR_		rjsrr[64];
	volatile LJURR_		ljurr[64];
				
} QADCREG;

extern void     TPUinit();
extern void     ADCIntrHndlr();

#endif	/*	__adc555_H */