/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


##############################################################################
#	init.s
#	------
#	Axiom Utilities for the Motorola MPC555 Development Board
#	Initialization file to boot from flash
#
#	Assembler:		GNU As
#
#	(C) Copyright 1999 Axiom Manufacturing.  All rights reserved.
#
#	ported to gnu as	Oct 2000	Peter Siebold
##############################################################################


	.file	"crt0.s"

    .align	2


##############################################################################
#	Stack Frame Definition
#		These Stack frame definitions have to be congruent with the 
#		Stack Frame usage in bseppcc.c
##############################################################################
	.equ	XR1,0
	.equ	XBLK,XR1+4
	.equ	XR0,XBLK+4
	.equ	XSRR0,XR0+4
	.equ	XSRR1,XSRR0+4
	.equ	XCTR,XSRR1+4
	.equ	XXER,XCTR+4
	.equ	XCR,XXER+4
	.equ	XLR,XCR+4
	.equ	XR2,XLR+4
	.equ	XR3,XR2+4
	.equ	XR4,XR3+4
	.equ	XR5,XR4+4
	.equ	XR6,XR5+4
	.equ	XR7,XR6+4
	.equ	XR8,XR7+4
	.equ	XR9,XR8+4
	.equ	XR10,XR9+4
	.equ	XR11,XR10+4
	.equ	XR12,XR11+4
	.equ	XR13,XR12+4
	.equ	XR14,XR13+4
	.equ	XR15,XR14+4
	.equ	XR16,XR15+4
	.equ	XR17,XR16+4
	.equ	XR18,XR17+4
	.equ	XR19,XR18+4
	.equ	XR20,XR19+4
	.equ	XR21,XR20+4
	.equ	XR22,XR21+4
	.equ	XR23,XR22+4
	.equ	XR24,XR23+4
	.equ	XR25,XR24+4
	.equ	XR26,XR25+4
	.equ	XR27,XR26+4
	.equ	XR28,XR27+4
	.equ	XR29,XR28+4
	.equ	XR30,XR29+4
	.equ	XR31,XR30+4
	.equ	XMSR,XR31+4
	.equ	STACK_FRAME_SIZE,XMSR+4


##############################################################################

	.ORG 0x0000								# location of the device to be
											# programed with this code 

	gInterruptVectorTable:


##############################################################################

	.ORG 0x00100


#	x0100 System Reset
#	This is the first instruction to be executed after the hardware power-on
#	reset.


		b	__start							# use this when when debugging
											# under the bdm orbooting in
											# single-chip (CMF)

#
#		b	(__start+0x400000)				# use this for the onboard flash version
#


##############################################################################
#
#	.ORG 0x00200
#	x0200 Machine Check
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00300
#	x0300 Data Storage
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00400
#	x0400 Instruction Storage
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
	.ORG 0X00500
#	x0500 External Interrupt
#	Override the behavior by over-writing the handler code in RAM.
  
	b IntrHndlr	
#
##############################################################################
#
#	.ORG 0X00600
#	x0600 Alignment
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00700
#	x0700 Program
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00800
#	x0800 Floating Point Unavailable
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00900
#	x0900 Decrementer Interrupt
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00C00
#	x0C00 System Call
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00D00
#	x0D00 Trace
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00E00
#	x0E00 FP Assist
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X00F00
#	x0F00 PPC740 and PPC750: Performance Monitor
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01000
#	x1000 	MPC8xx and MPC505:	Software Emulation
#			PPC603e:			Instruction TLB Miss
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01100
#	x1100 	MPC8xx:				Instruction TLB Miss
#			PPC603e:			Data Load TLB Miss
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01200
#	x1200	MPC8xx:				Data TLB Miss
#			PPC603e:			Data Store TLB Miss
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01300
#	x1300	MPC8xx:				Instruction TLB Error
#			PPC7xx and PPC603e:	Instruction address breakpoint
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01400
#	x1400	MPC8xx:				Data TLB Error
#			PPC7xx and PPC603e:	System management
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01c00
#	x1C00 MPC8xx and MPC505:	Data breakpoint
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01d00
#
#	x1D00 MPC8xx and MPC505:	Instruction breakpoint
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#	.ORG 0X01e00
#
#	x1E00 MPC8xx and MPC505:	Peripheral breakpoint
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################
#
#	.ORG 0X01f00
#	x1F00 MPC8xx and MPC505:	Non-maskable development port
#	Override the behavior by over-writing the handler code in RAM.
#
##############################################################################


.ORG 0x02000 

__start:
start:

hwreset:

##############################################################################
#
# If using Hardware floating point enable the FP bit in the  MSR */
# 	mfmsr 0
#	ori  0,0,0x00002000
#	mtmsr 0
#
##############################################################################


	addis           %r11,%r0,__SP_INIT@ha	# Initialize stack pointer r1 to
	addi            %r1,%r11,__SP_INIT@l	# value in linker command file.
	addis           %r13,%r0,_SDA_BASE_@ha	# Initialize r13 to sdata base
	addi            %r13,%r13,_SDA_BASE_@l	# (provided by linker).
	addis           %r2,%r0,_SDA2_BASE_@ha	# Initialize r2 to sdata2 base
	addi            %r2,%r2,_SDA2_BASE_@l	# (provided by linker).
	addi            %r0,%r0,0				# Clear r0.


	stwu			%r0,-64(%r1)			# Terminate stack.


	LIS				%r0,0XFFF0 				# IMMR
	ori				%r0,%r0,0x800
	mtspr			638,%r0

	li				%r0,0x0000				# BBCMCR
	mtspr			560,%r0

	bl				inithw					# initialize hardware registers

	bl              __init_main				# Finishes initialization (copies .data
                                        	# ROM to RAM, clears .bss), then calls
                                        	# example main(), which calls exit(),
                                       		# which halts.

#########################################################################
#	Generic Interrupt Handler
#	void IntrHndlr (void)
#########################################################################

IntrHndlr:
#Save context
	stwu	%r1,-STACK_FRAME_SIZE(%r1)
	stw		%r0,XR0(%r1)
	mfspr	%r0,%LR
	stw		%r0,XLR(%r1)
	mfcr	%r0
	stw		%r0,XCR(%r1)
	mfspr	%r0,%XER
	stw		%r0,XXER(%r1)
	mfspr	%r0,%CTR
	stw		%r0,XCTR(%r1)
	mfspr	%r0,%SRR1
	stw		%r0,XSRR1(%r1)
	mfspr	%r0,%SRR0
	stw		%r0,XSRR0(%r1)
	mfmsr	%r0
	stw		%r0,XMSR(1)
	stw		%r2,XR2(%r1)
	stw		%r3,XR3(%r1)
	stw		%r4,XR4(%r1)
	stw		%r5,XR5(%r1)
	stw		%r6,XR6(%r1)
	stw		%r7,XR7(%r1)
	stw		%r8,XR8(%r1)
	stw		%r9,XR9(%r1)
	stw		%r10,XR10(%r1)
	stw		%r11,XR11(%r1)
	stw		%r12,XR12(%r1)
	stw		%r13,XR13(%r1)
	stw		%r14,XR14(%r1)
	stw		%r15,XR15(%r1)
	stw		%r16,XR16(%r1)
	stw		%r17,XR17(%r1)
	stw		%r18,XR18(%r1)
	stw		%r19,XR19(%r1)
	stw		%r20,XR20(%r1)
	stw		%r21,XR21(%r1)
	stw		%r22,XR22(%r1)
	stw		%r23,XR23(%r1)
	stw		%r24,XR24(%r1)
	stw		%r25,XR25(%r1)
	stw		%r26,XR26(%r1)
	stw		%r27,XR27(%r1)
	stw		%r28,XR28(%r1)
	stw		%r29,XR29(%r1)
	stw		%r30,XR30(%r1)
	stw		%r31,XR31(%r1)

	bl	  Intr_Hdlr

	lwz		%r2,XR2(%r1)
	lwz		%r3,XR3(%r1)
	lwz		%r4,XR4(%r1)
	lwz		%r5,XR5(%r1)
	lwz		%r6,XR6(%r1)
	lwz		%r7,XR7(%r1)
	lwz		%r8,XR8(%r1)
	lwz		%r9,XR9(%r1)
	lwz		%r10,XR10(%r1)
	lwz		%r11,XR11(%r1)
	lwz		%r12,XR12(%r1)
	lwz		%r13,XR13(%r1)
	lwz		%r14,XR14(%r1)
	lwz		%r15,XR15(%r1)
	lwz		%r16,XR16(%r1)
	lwz		%r17,XR17(%r1)
	lwz		%r18,XR18(%r1)
	lwz		%r19,XR19(%r1)
	lwz		%r20,XR20(%r1)
	lwz		%r21,XR21(%r1)
	lwz		%r22,XR22(%r1)
	lwz		%r23,XR23(%r1)
	lwz		%r24,XR24(%r1)
	lwz		%r25,XR25(%r1)
	lwz		%r26,XR26(%r1)
	lwz		%r27,XR27(%r1)
	lwz		%r28,XR28(%r1)
	lwz		%r29,XR29(%r1)
	lwz		%r30,XR30(%r1)
	lwz		%r31,XR31(%r1)
	lwz		%r0,XLR(%r1)
	mtspr	%LR,%r0
	lwz		%r0,XCR(%r1)
	mtcrf	255,%r0
	lwz		%r0,XXER(%r1)
	mtspr	%XER,%r0
	lwz		%r0,XCTR(%r1)
	mtspr	%CTR,%r0
	lwz		%r0,XSRR1(%r1)
	mtspr	%SRR1,%r0
	lwz		%r0,XSRR0(%r1)
	mtspr	%SRR0,%r0
	lwz		%r0,XMSR(%r1)
	mtmsr	%r0
	lwz		%r0,XR0(%r1)
	addi	%r1,%r1,STACK_FRAME_SIZE
	rfi

	.end
