/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/*----------------------------------------------------------------------------------**
** Copyright (c) 2000, Peter Siebold / Jeff Clark                                   **
** All rights reserved.                                                             **
** Redistribution and use in source and binary forms, with or without modification, **
** are permitted provided that the following conditions are met:                    **
**                                                                                  **
**                                                                                  **
** Redistributions of source code must retain the above copyright notice, this list **
**  of conditions and the following disclaimer.                                     **
**                                                                                  **
** Redistributions in binary form must reproduce the above copyright notice, this   **
** list of conditions and the following disclaimer in the documentation and/or      **
** other materials provided with the distribution.                                  **
**                                                                                  **
** Neither the name of "aems555" nor the names of its contributors may be used      **
** to endorse or promote products derived from this software without specific prior **
** written permission.                                                              **
**                                                                                  **
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''    **
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE        **
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   **
** DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY      **
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES       **
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     **
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON   **
** ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          **
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS    **
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                     **
**----------------------------------------------------------------------------------**
**																					**
**----------------------------------------------------------------------------------**
**    EFI code for the ppc555														**
**																					**
**	File:		hw.h																**
**	Desc:		Hardware Support for Axiom Manufacturing PB0555 Development Board	**
**																					**
**																					**
**	Author:		Peter Siebold														**
**	Tools:   	GNU tool chain														**
**																					**
**----------------------------------------------------------------------------------*/

#ifndef	__HW_h
#define	__HW_h


/* Define byte, word, and long word types
*/
typedef unsigned char u_char; 
typedef unsigned short int u_short;
typedef unsigned long u_long;
typedef unsigned int u_int; 


/* fixed point stuff
*/
typedef long fixed;						/* Our new fixed point type.	*/
#define itofx(x) ((x) << 8)				/* Integer to fixed point		*/
#define ftofx(x) ((x) * 256)			/* Float to fixed point			*/
#define dtofx(x) ((x) * 256)			/* Double to fixed point		*/
#define fxtoi(x) ((x) >> 8)				/* Fixed point to integer		*/
#define fxtof(x) ((float) (x) / 256)	/* Fixed point to float			*/
#define fxtod(x) ((double)(x) / 256)	/* Fixed point to double		*/
#define Mulfx(x,y) (((y) * (x)) >> 8)	/* Multiply a fixed by a fixed	*/
#define Divfx(x,y) ((y << 8) / (x))    	/* Divide a fixed by a fixed	*/


/* clock speeds, interrupt vector offsets and levels, arbitration values
68332
#define	FCRYSTAL	32768
#define	SYSCLK_W	0
#define	SYSCLK_X	1
#define	SYSCLK_Y	63
#define SYSCLK		(FCRYSTAL * 4 * (SYSCLK_Y+1) * (1 << (2 * SYSCLK_W + SYSCLK_X))) */

// mpc555
#define OSCCLK	4000000									// External Crystal Rate	Mhz
#define DIVF	0										// Division Factor			Not Currently implemented, keep at 0	
#define MF		9										// Multiplication Factor
#define RTDIV	0										// RTCLOCK DIV FACTOR 0 IS 4 1 IS 256
#define SYSCLK OSCCLK / ( DIVF + 1 ) * ( MF + 1 ) * 2	//Mhz


#define SCI1_BAUD            9600       /* 57.6 kbps                            */
#define SCI2_BAUD            57600       /* 57.6 kbps                            */
#define QSPI_BAUD           209000      /* 209 kHz                              */
#define PIT_RATE            75	      	/* periodic interrupt timer rate        */

//For 332
//#define scidivider(baud)  (((SYSCLK >> 5)+((long)(baud)/2))/(baud))
#define qspidivider(baud) (((SYSCLK >> 1)+((long)(baud)/2))/(baud))
//For 555
#define scidivider(baud)  SYSCLK/32/baud 

#if RTDIV
#define PITC(freq)  (OSCCLK / 256 / freq) - 1
#else
#define PITC(freq)  (OSCCLK / 4 / freq) - 1
#endif

//#define SIMBASE	0xFFFA00  
#define RAMBASE		0x300000	//Configured for the 555  
#define QSMBASE		0x305000	//Configured for the 555  
//#define QSMDATABASE	0xFFFD00  
#define TPUREGBASE	0x304000	//Configured for the 555  
#define TPUDPBASE		0x304100	//Configured for the 555	  
#define EMURAMBASE	0x302000	//Configured for the 555

#define QADCREGBASE	0x304800	//Configured for the 555 


//#define VECTORBASE	0xB0000     /* Vector Base Register value */ 	Only for 332?
//#define SCIVEC		0x56		/* SCI receive vector number  */
//#define QSPIVEC		0x57        
//#define TPUVEC		0x58    
//#define PITVEC		0x40

/*#define SCI_IRQL            0x6						Only for 332?
#define PIT_IRQL            0x7
#define QSPI_IRQL           0x6
#define TPU_IRQL            0x6*/

#define	PIT_IRQL		0x00
#define	TPU_IRQL		0x01
#define	QADC_IRQL	0x02		//configured for the 555
#define	SCI_IRQL		0x03
#define	QSPI_IRQL	0x03

//#define QSM_IARB            0x7  
//#define TPU_IARB            0xC
//#define PIT_IARB            0x5


unsigned short addata[32];

u_char		PMMXSync;


#define CJCGAINRATIO 135	/* Cold junk 40.75 uV/degC with a gain of 130 in fixed point */
	
#define CHT1GAINRATIO 74	/* amp gain of 446.3 corrected to 130 in fixed point */
#define CHT2GAINRATIO 83	/* amp gain of 397.7 corrected to 130 in fixed point */
#define CHT3GAINRATIO 71	/* amp gain of 466 corrected to 130 in fixed point */
#define CHT4GAINRATIO 79	/* amp gain of 418 corrected to 130 in fixed point */
#define EGT1GAINRATIO 247	/* amp gain of 134.7 corrected to 130 in fixed point */
#define EGT2GAINRATIO 243	/* amp gain of 136.4 corrected to 130 in fixed point */
#define EGT3GAINRATIO 264	/* amp gain of 126 corrected to 130 in fixed point */
#define EGT4GAINRATIO 247	/* amp gain of 134.7 corrected to 130 in fixed point */

#define CJC		20

#define CHT1	1
#define CHT2	3
#define CHT3	5
#define CHT4	7
#define EGT1	9
#define EGT2	11
#define EGT3	13
#define EGT4	15

#define VOLT	22


#define MAT		9
#define MAP		10

#define AMP		12
#define OILP	13
#define OILT	14
#define FUElP	15
#define TPS		16
#define BARO	17
#define RPM		18
#define ADV		19
#define FF1		20
#define FF2		21
#define FF3		22
#define FF4		23

typedef struct
{

	u_short Header;
	u_short Cjc;
	u_short Egt1;
	u_short Egt2;
	u_short Egt3;
	u_short Egt4;
	u_short Cht1;
	u_short Cht2;
	u_short Cht3;
	u_short Cht4;
	u_short Mat;
	u_short Map;
	u_short Volt;
	u_short Amp;
	u_short OilP;
	u_short OilT;
	u_short FuelP;
	u_short Tps;
	u_short Baro;
	u_short Rpm;
	u_short Adv;
	u_short Ff1;
	u_short Ff2;
	u_short Ff3;
	u_short Ff4;
	u_short spare1;
	u_short spare2;
	u_short spare3;
	u_short spare4;
	u_short spare5;
	u_char status1;
	u_char status2;
	u_char status3;
	u_char checksum;

}TMBUFF;


#endif
