/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/*----------------------------------------------------------------------------------**
** Copyright (c) 2000, Peter Siebold                                                **
** All rights reserved.                                                             **
** Redistribution and use in source and binary forms, with or without modification, **
** are permitted provided that the following conditions are met:                    **
**                                                                                  **
**                                                                                  **
** Redistributions of source code must retain the above copyright notice, this list **
**  of conditions and the following disclaimer.                                     **
**                                                                                  **
** Redistributions in binary form must reproduce the above copyright notice, this   **
** list of conditions and the following disclaimer in the documentation and/or      **
** other materials provided with the distribution.                                  **
**                                                                                  **
** Neither the name of "aems555" nor the names of its contributors may be used      **
** to endorse or promote products derived from this software without specific prior **
** written permission.                                                              **
**                                                                                  **
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''    **
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE        **
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   **
** DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY      **
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES       **
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     **
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON   **
** ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          **
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS    **
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                     **
**----------------------------------------------------------------------------------**
**																					**
**----------------------------------------------------------------------------------**
**    EFI code for the ppc555														**
**																					**
**	File:		main.c																**
**	Desc:		Support for the MPC555 powerend engine managment system for a		**
**				Lycoming O-360														**
**																					**
**																					**
**	Author:		Peter Siebold														**
**	Tools:   	GNU tool chain														**
**																					**
**----------------------------------------------------------------------------------*/

 
/*
By undefining the following, you remove all support for those subsystems from the
code for debugging.  This results in smaller code size and reduced downloading times
to the processor*/

#define TPU 
#define ADC 
#define QSM 
 
#include "hw.h" 
 
#ifdef TPU 
#include "tpu.h" 
#endif 
 
#ifdef ADC 
#include "adc.h" 
#endif 
 
#ifdef QSM 
#include "qsm.h" 
#endif 
 
#include "gnu.h" 
#include "serial.h" 
 
TMBUFF tm; 
volatile long frameCount = 0;
 
void delay(); 
u_char CheckSum( u_char* buffer, u_short size); 
void PITCInit(void);

unsigned int rpm; 
u_short tpustat; 
 
u_short skip;

#ifdef TPU

u_char PMMXSync;

u_char tooth;
u_char synch;
u_char stall;
u_char shortgap;
u_char noise;
u_char syncstall;
u_char timeout;
u_char other;
u_char tpureset;

unsigned int ramerror1;
unsigned int ramerror2;

#endif

#ifdef ADC

#include "counts2degF.h"
extern const u_short count2mV[];

#endif 
 
__main()
{ 
 
	u_char choice; 
	char tempString[20]; 
	unsigned long prevFrame; 
 
	fixed cjcfxCount;
	u_short tcCount;
	u_short result;


	rpm = 0; 

	skip = 0;
 
	tm.Header = (u_short)0x4040; 
 
	delay();	// delay in case hardware needs time 
 
	/* 
	SCI port one is utilized as a debug port, and is implemented with the
	default serial port drivers from the axiom hello world example.  SCI2
	utilizes an interrupt driven driver ported for a Motorola example.
	*/

	InitCom(0);	// initialize SCI1 serial port 1 
//	InitCom(1);	// initialize SCI2 serial port 2 
	ioport = 0;	// default to 0 for now, don't detect. 
 
	delay();	// delay in case hardware needs time
 
	/*
	This section was a late addition to the program.  I was chasing a bug that
	seemed to be related to a hardware RAM error.  Turned out to be a bug in
	the CheckSum computation function.  This code checks weather the ram check
	routine in crto.c reported any errors.  It must be done here after the
	serial port has been initialized and can send out the results.  Nice bit,
	but probably not needed.
	*/

	if(ramerror1 != 0 || ramerror2 != 0)
	{
		puts2("RAMERROR RAMERROR ");
		itoa(ramerror1, tempString);
		puts2(tempString);
		puts2(" ");
		itoa(ramerror2, tempString);
		puts2(tempString);
        puts2(" bytes\n");
		/*
		right now this is a fatal error, and will halt here.  Maybe should just
		set a check engine lite....
		*/
		while(1);
	}	


	puts2("\n Hello World on port 1!");
	puts2("\nRAM check OK");	
 
#ifdef TPU 
	puts2("\n Starting TPU Init!"); 
	TPUinit(); 
	puts2("\n Done with TPU Init!"); 
#endif 
 
#ifdef ADC	 
	puts2("\n Starting ADC Init!"); 
	ADCInit(); 
	puts2("\n Done with ADC Init!"); 
#endif 
 
#ifdef QSM 
	puts2("\n Starting SCI Init!"); 
	SCIinit(); 
	puts2("\n Done with SCI Init!"); 
#endif

	/*
	Sets up Periodic Interrupt Timer
	*/
	PITCInit();


	/*
	Sets up interrupt support
	*/
 
    put32(0x2fc014, 0xFFFF0000);    //SIU Interrupt Mask Register SIMASK 
                                    //Currently set for all levels.  
		 
	/*
	mtspr 80, %r0 enables interrupts.  See AN1281
	*/

	puts2("\n Enableing Interrupts!\n\n");
	/*Enables external interrupts 80 is equiv to EIE */
	asm ("mtspr 80, %r0");

 
	/* This enters an infanite loop, never to return.......... */

	for(; ;)
	{ 
 
		/*
		This throttles the main loop rate.  The PITC has been defined to increment
		frameCount at 50 Hz.
		*/
		
		frameCount = prevFrame; 

		while(frameCount == prevFrame);

#ifdef TPU

		tm.Rpm = (u_short)TPUgetRpm();
		tm.Mat = (u_short)TPUgetPeriod();
		tm.Map = (u_short)TPUgetLongperiod();

		tm.Amp = TPUgetCOUNT_360();

		/*
		if engine RPM is greater than 300, then advance the timeing to 25 BTDC
		else keep at 0 deg for starting.  The assumption is that the engine
		can't run less than 300 RPM
		*/

		if(tm.Rpm > 300)
		{
			TPUsparkadj(itofx(25));
			tm.Adv = 25;
		}
		else
		{
			TPUsparkadj(itofx(0));
			tm.Adv = 0;
		}

		tm.status1 = PMMXSync;


#endif	//TPU

#ifdef ADC


		cjcfxCount =
			(fixed)(addata[CJC] * CJCGAINRATIO);	/* Now fixed point,		*/
		tm.Cjc  = count2mV[addata[CJC]];			/* GAINRATIO is fixedpt */

		tcCount = fxtoi(
			(fixed)(addata[CHT1] * CHT1GAINRATIO)	/* Now fixed point,		*/
		+ cjcfxCount);								/* GAINRATIO is fixedpt */
		tm.Cht1 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[CHT2] * CHT2GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Cht2 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[CHT3] * CHT3GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Cht3 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[CHT4] * CHT4GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Cht4 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[EGT1] * EGT1GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Egt1 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[EGT2] * EGT2GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Egt2 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[EGT3] * EGT3GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Egt3 = count2degF[tcCount & 0x03FF];

		tcCount = fxtoi(
		(fixed)(addata[EGT4] * EGT4GAINRATIO)		/* Now fixed point,		*/
		+ cjcfxCount								/* GAINRATIO is fixedpt */
		);
		tm.Egt4 = count2degF[tcCount & 0x03FF];

		tm.Volt = (u_short)(count2mV[addata[VOLT]] * 7);

		/*
		The following are not implemented in this verion of the efi code.
		They are shown here only as a reminder for the next version.
		
		
		tm.Mat = addata[MAT];
		tm.Map = addata[MAP];
		tm.Volt = addata[VOLT];
		tm.Amp = addata[AMP];
		tm.OilP = addata[OILP];
		tm.OilT = addata[OILT];
		tm.FuelP = addata[FUElP];
		tm.Tps = addata[TPS];
		tm.Baro = addata[BARO];
		tm.Rpm = addata[RPM];
		tm.Adv = addata[ADV];
		tm.Ff1 = addata[FF1];
		tm.Ff2 = addata[FF2];
		tm.Ff3 = addata[FF3];
		tm.Ff4 = addata[FF4];

		*/
		

#endif	//ADC 

		tm.spare1 =  ((u_short)tooth << 8) | ((u_short)synch);
		tm.spare2 =  ((u_short)stall << 8) | ((u_short)shortgap);
		tm.spare3 =  ((u_short)noise << 8) | ((u_short)syncstall); 
		tm.spare4 =  ((u_short)timeout << 8) | ((u_short)other);		
		tm.spare5 =  ((u_short)tpureset << 8);

		tm.checksum = CheckSum((void*)&tm, sizeof(TMBUFF)-1);

#ifdef QSM  
		SCIPutBytes((void*)&tm, sizeof(TMBUFF));
#endif //QSM

	}	/* End of infinite for loop */

 
} /* End of __main function */


/*---------------------------------------------------------------------------*/
/*  Calculate 1 byte checksum for TM structure
	called once per frame of main loop
*/

u_char CheckSum( u_char* buffer, u_short size) 
{ 
	u_short i; 
	u_char checksum; 

	checksum = 0;
 
	for(i=0;i<size;i++) 
	{
		/* checksum =+ buffer[i]; does not work?! */

		checksum = checksum + buffer[i]; 
	} 

	return checksum; 
}  /* End of function CheckSum */

/*---------------------------------------------------------------------------*/
/*  Generic delay routine.  Copied from Axiom hello world demo program
	called after setting up serial ports to let hardware settle before using
*/

void delay(void)
{ 
	unsigned long delaycount; 
 
	delaycount=0; 

	do
	{ 
		delaycount++; 
	}while(delaycount < 1000); 

} /* End of function delay */
 
/*---------------------------------------------------------------------------*/
/*	INITIALIZE HARDWARE REGISTERS 
	This routine is called once following RESET from 
*/

void inithw(void)
{ 
	unsigned long *Rptr; 
	unsigned short *sRptr; 
 
	unsigned int temp1, temp2; 
 
	put32(0x2fc000,0x420800);	// SIUMCR make irq's general purpose i/o 
 
//	put32(0x2fc000,0x420000);	// SIUMCR make irq's irq, NOT TESTED 
 
    put32(0x2fc004,0x00FF88);	// SYPCR 
 
	put32(0x2fc100,0x400003);	// BR0 32-bit set here for BDM or external flash 
	put32(0x2fc104,0xfff80022);	// OR0 2 wait states 
//	put32(0x2fc104,0xfff80012);	// OR0 1 wait state 
//	put32(0x2fc104,0xfff800f2);	// OR0 15 wait states 
 
	put32(0x2fc108,0x800003);	// BR1 
	put32(0x2fc10c,0xfff80002);	// OR1 
	put32(0x2fc110,0xc00003);	// BR2 
	put32(0x2fc114,0xfffe0022);	// OR2 
	put32(0x2fc118,0x01000403);	// BR3 Keypad/LCD 
	put32(0x2fc11c,0xfffe0FF1);	// OR3 maximum wait states 
	put32(0x2fc140,0x00000000);	// DMBR 
	put32(0x2fc144,0x00000000);	// DMOR 
	put32(0x2fc280,0x10000 | (RTDIV << 25));	// SCCR 
 
//	put32(0x2fc280,0x10000);	// SCCR 
 
	//should start using MF here defined in hw.h 
 
//	put32(0x2fc284,0x00400000);	// PLPRCR 20mhz 
//	put32(0x2fc284,0x00900000);	// PLPRCR 40mhz 
	put32(0x2fc284, MF << 20);	// PLPRCR 40mhz 
 
	put16(0x300000,0x00);	// DPT 
	put16(0x300004,0xffa0);	// RAM base addr 
 
	put16(0x305014,0x00);	// PORTQS 
	put16(0x305016,0x00);	// PQSPAR/DDRQS 
	put16(0x306100,0x00);	// MPIOSMDR 
	put16(0x306102,0x00);	// MPIOSMDDR 
	put16(0x306800,0x00);	// MPIOS1TPCR 
	put16(0x380000,0x00);	// SRAMMCR 
 
	put32(0x2fc024,0x00);	// SGPIODT1 
	put32(0x2fc028,0x00);	// SGPIODT2 
	put32(0x2fc02c,0x00);	// SGPIOCR 
	put32(0x2fc030,0x00);	// EMCR
 
	put32(0x2fc03c,0x02000000);	// PDMCR		disable pad pull ups and downs

	put32(0x307f80,0x00);	// UMCR 
 
 
} /* End of function inithw */

/*---------------------------------------------------------------------------*/
/*	INITIALIZE Periodic Interrupt Controller 
	This routine is called once during start up from functin __main()
*/

void PITCInit(void)
{
	/*put16(0x2fc244, 0xFFFF);*/	/* PITC Count Register */
									/* This set the count register to the max */ 
 
    put16(0x2fc244, PITC(PIT_RATE));/* PITC Count Register  */
                                    /* This set the count register to the max */
 
    put16(0x2fc240, 0x8005);		/* PISCR Count Register */
									/* Level 0 Interrupt Priority  */
									/* This configures the PIC timer  */
 
} /* End of function PITCInit */

/*---------------------------------------------------------------------------*/
/*	Periodic Interrupt Controller 
	This function is called during a PITC interrupt from Intr_Hdlr()
*/

void PITCIntrHndlr(void) 
{ 

	put16(0x2fc240, 0x8085);		/* PITC Count Register  */
									/* Level 0 Interrupt Priority  */
                                	/* This writes a 1 to the PS
									/* in the PITC register */
									/* and clears interrupt */

	frameCount++;					/* increments frameCount, now main loop
									/* can continue */

} /* End of function PITCIntrHndlr */ 
 
 
/*---------------------------------------------------------------------------*/
/*	Interrupt Handler 
	This function is called during all interupts from IntrHndlr in crt0.s
	after all general purpose registers have been saved on the stack.  When
	this function returns, all registers are restored.
*/
void Intr_Hdlr (void) 
{ 
//	char inttmpString[20]; 
	unsigned short i;
	unsigned int beforeinterrupt; 
	//unsigned short beforeinterrupt; 
	unsigned int sourceinterrupt;
 
	asm ("	mtspr	81,%r0");   // Interrupts are already disabled, but this 
								// sets the ri bit in the msr 
 
	beforeinterrupt = get16(0x2FC010);
	
	/*
	This was a test, but never implemented, but should work.  the cntlzw
	function counts the number of 0 starting at the MSB of beforeinterrupt
	until it reaches a 1.  Then places the count into i.  This could then
	be used as a offset into an array of interrupt handlers to call.  I'll
	implement on the next version
	

	asm("cntlzw %1, %0" : "=r"(i) : "r"(beforeinterrupt));

	*/

	/*
	this is a very crude way of doing the interrupts, but without a
	reliable BDM debugger, it was easier to debug this.  In the next
	version I will implement the method mentioned above.
	*/

	if((beforeinterrupt & 0x4000) == 0x4000)	/* if Level 0 interrupt */
	{ 
		/* Level 0 */
		PITCIntrHndlr();
		/* puts2("."); */
	} 
 
	else if((beforeinterrupt & 0x1000) == 0x1000)	 // if Level 1 interrupt 
	{	// Level 1 
#ifdef TPU 
		TPUIntrHndlr(); 
#endif 
//		htoa(beforeinterrupt, inttmpString); 
//		puts2(inttmpString); 
//		puts2("\n"); 
	} 
	else if((beforeinterrupt & 0x0400) == 0x0400)	 // if Level 2 interrupt 
	{	// Level 2 
//		htoa(beforeinterrupt, inttmpString); 
//		puts2(inttmpString); 
#ifdef ADC 
		ADCIntrHndlr(); 
#endif 
//		puts2("."); 
	} 
	else if((beforeinterrupt & 0x0100) == 0x0100)	 // if Level 3 interrupt 
	{	// Level 3 
//		htoa(beforeinterrupt, inttmpString); 
//		puts2(inttmpString); 
#ifdef QSM 
		sciint(); 
#endif 
//		puts2("."); 
	} 
	else puts2("?!"); //Unknown interrupt.  Prints ?! to the debug serial port.
	 
 
	/*
	re-enable interrupts gets handled when rfi is called in the crt0.s
	generic interrupt handler 
	*/


} /* End of function Intr_Hdlr */ 
 

 
 
