/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/* 	qsm.h 
*/


#ifndef __QSM_h
#define	__QSM_h

/* Defines 
*/ 
#define qsize 160						/* Queue size - bytes */

/* Typedef for tx and rx queue structures for sci buffer routines
*/ 
typedef struct { 
	u_short in;        				 	/* Queue input/write pointer */ 
	u_short full;      				 	/* Queue full flag */
	u_short out;       				 	/* Queue output/read pointer */ 
	u_char q[qsize];  				 	/* Queue array */
} queue_struct;


struct QSMCR_ {
	volatile u_short	STOP:1;
	volatile u_short	FRZ:1; 			//Modified for 555, was 2 bits
	u_short	:6;							//Modified for 555, was 5 bits
	volatile u_short	SUPV:1;
	u_short	:3;
	volatile u_short	IARB:4;
};
                    
struct QSCIILR_ {
	u_char	:3;							//Modifed for 555, was 2 bits
//	volatile u_char		ILQSPI:3;	//Removed for 555, moved in this implementation
	volatile u_char		ILSCI:5;
	u_short	:8;							//Modified for 555, was 3 bits
};

struct	QSPIILR_{						//Added for 555
	u_short	:11;							//reserved
	volatile u_char		ILQSPI:5;	
};

struct SCCR0_ {
	volatile u_char		OTHR:1;		//Added for 555, baud clock source select
	volatile u_char		LNKBD:1;		//Added for 555, link baud to sci1
	u_short :1;							   //Modifed for 555, was 3 bits
	volatile u_short	SCBR:13;
};

struct SCCR1_ {
	u_short	:1;
	volatile u_short	LOOPS:1;
	volatile u_short	WOMS:1;
	volatile u_short	ILT:1;
	volatile u_short	PT:1;
	volatile u_short	PE:1;
	volatile u_short	M:1;
	volatile u_short	WAKE:1;
	volatile u_short	TIE:1;
	volatile u_short	TCIE:1;
	volatile u_short	RIE:1;
	volatile u_short	ILIE:1;
	volatile u_short	TE:1;
	volatile u_short	RE:1;
	volatile u_short	RWU:1;
	volatile u_short	SBK:1;
};

struct SPCR_ {
	/* now spcr0 */
	volatile u_short	MSTR:1;
	volatile u_short	WOMQ:1;
	volatile u_short	BITS:4;
	volatile u_short	CPOL:1;
	volatile u_short	CPHA:1;
	volatile u_short	SPBR:8;
	/* now spcr1 */
	volatile u_short	SPE:1;
	volatile u_short 	DSCKL:7;
	volatile u_short 	DTL:8;
	/* now spcr2 */
	volatile u_short	SPIFIE:1;
	volatile u_short	WREN:1;
	volatile u_short	WRT0:1;
	volatile u_short	ENDQP:5;			//Modified for 555, was 4 bits
	u_short	:3;							//Modified for 555, was 4 bits
	volatile u_short	NEWQP:5;			//Modified for 555, was 4 bits
	/* now spcr3 */
	u_short	:5;
	volatile u_short	LOOPQ:1;
	volatile u_short	HMIE:1;
	volatile u_short	HALT:1;
	volatile u_short	SPIF:1;
	volatile u_short	MODF:1;
	volatile u_short	HALTA:1;
	volatile u_short	CPTQP:5;			//Modified for 555, was 4 bits
};

/*
 * sr must not be defined as bitset, as it must be accessed in a single
 * read. Bitset operations are implemented by read-and-mask, so bits may
 * get masked out without being able to look at. At the next read the 
 * state might be gone.
 */
#define		SCSR_TDRE	(1<<8)		/* transm data reg empty */
#define		SCSR_TC		(1<<7)		/* transm complete flag */
#define		SCSR_RDRF	(1<<6)		/* rec data reg full */
#define		SCSR_RAF		(1<<5)		/* rec active */
#define		SCSR_IDLE	(1<<4)		/* line idle */
#define		SCSR_OR		(1<<3)		/* overrun */
#define		SCSR_NF		(1<<2)		/* noise */
#define		SCSR_FE		(1<<1)		/* framing error */
#define		SCSR_PF		(1<<0)		/* parity failed */

struct CR_ {
	volatile u_char	CONT:1;
	volatile u_char	BITSE:1;
	volatile u_char	DT:1;
	volatile u_char	DSCK:1;
	volatile u_char	PCS:4;
};

typedef struct qsm {
	struct 	QSMCR_ 			qsmcr;						//Modified for 555
	volatile u_short			qtest;
	struct	QSCIILR_			qsciilr;						//Modified for 555
//	volatile u_char			qivr;							//Removed for 555
	struct	QSPIILR_			qspiilr;						//Added for 555
	struct	SCCR0_			scc1r0;						//Modified for 555
	struct	SCCR1_			scc1r1;
	volatile u_short			sc1sr;
	volatile u_short			sc1dr;
	volatile u_short			qreserved3;
	volatile u_short			qreserved4;
	volatile u_char			qreserved5;
	volatile u_char			portqs;
	volatile u_char			pqspar;						//if used, then must modify for 555
	volatile u_char			ddrqs;						//if used, then must modify for 555
	struct SPCR_ 				spcr;

	struct	SCCR0_			scc2r0;						//Added for 555
	struct	SCCR1_			scc2r1;
	volatile u_short			sc2sr;
	volatile u_short			sc2dr;

	volatile u_short			qreserved6[0x6C];
//	volatile u_short			qreserved6[0x70];			//Changed and added second sci above
	volatile u_short			rr[0x10];
	volatile u_short			tr[0x10];
	struct CR_					cr[0x10];
} QSMREG;



/* Global Variables 
*/ 
queue_struct rxbuff, txbuff;			/* Transmit and receive queue structures */
unsigned char locbuf[10];

/* Function prototypes 
*/
extern void SCIPutS (char *message);
extern void SCIPutBytes(char *val, int len);
extern void qinit (queue_struct *sptr);		/* Initialize a specified buffer */
extern u_short qstat (queue_struct *sptr);		/* Return the status of a specified buffer */ 
extern void SCIinit ();                		/* SCI module initialization */
extern void QSPIinit ();                       /* QSPI module initialization */
extern void sciint ();             			/* SCI interrupt handler (TX and RX) */
extern char rx_byte (u_char *b);  		   		/* Read a byte from the RX buffer */
extern char tx_byte (u_char);            		/* Write a byte to the TX buffer */
extern void SCIPutInt(int num, int len);
extern void SCIPutHex(unsigned int num, int len);




#endif

