/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/*----------------------------------------------------------------------------------**
** Copyright (c) 2000, Peter Siebold / Jeff Clark                                   **
** All rights reserved.                                                             **
** Redistribution and use in source and binary forms, with or without modification, **
** are permitted provided that the following conditions are met:                    **
**                                                                                  **
**                                                                                  **
** Redistributions of source code must retain the above copyright notice, this list **
**  of conditions and the following disclaimer.                                     **
**                                                                                  **
** Redistributions in binary form must reproduce the above copyright notice, this   **
** list of conditions and the following disclaimer in the documentation and/or      **
** other materials provided with the distribution.                                  **
**                                                                                  **
** Neither the name of "aems555" nor the names of its contributors may be used       **
** to endorse or promote products derived from this software without specific prior **
** written permission.                                                              **
**                                                                                  **
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''    **
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE        **
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   **
** DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY      **
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES       **
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     **
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON   **
** ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          **
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS    **
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                     **
**----------------------------------------------------------------------------------**
**																					**
**----------------------------------------------------------------------------------**
**    EFI tpu code for the ppc555													**
**																					**
**	File:		tpu.c																**
**	Desc:		Support for the MPC555 TPU3 using PMMX, PMMA, ITC functions			**
**              This code had its beginnings from Jeff Clarks efi332 tpu code		**
**              but since been completely reworked for MPC555 and Lycoming O-360	**
**																					**
**																					**
**	Authors:	Peter Siebold / Jeff Clark											**
**	Tools:   	GNU tool chain														**
**																					**
**----------------------------------------------------------------------------------*/



#include "hw.h"
#include "utils.h"
#include "tpu.h"  
#include "emuram.h"

#include "serial.h"
        



/* overlay structure onto TPU registers
*/
TPUREG   *tpureg   = (TPUREG *)    TPUREGBASE;
TPUDPRAM *tpudpram = (TPUDPRAM *)  TPUDPBASE;
TRAMREG  *tramreg  = (TRAMREG *)   RAMBASE;

extern unsigned short rpm;
extern TMBUFF tm;
fixed TCR1freq;

u_short prevSyncStallTooth, currentSyncStallTooth;

unsigned int cmcount = 0;

extern u_char PMMXSync; 
extern u_char tooth;
extern u_char synch;
extern u_char stall;
extern u_char shortgap;
extern u_char noise;
extern u_char syncstall;
extern u_char timeout;
extern u_char other;
extern u_char tpureset;

void TPUinit()
{



    /*  the use of unions for the TPU register data structures is to enforce
        the word writes that are necessary to TPU registers.  Bitfield 
        operations as used for the other registers will result in byte writes and
        incorrect register setup.  Here temporary local copies of the registers 
        are allocated and setup, then copied as a whole to the true register.
    */      
	TPUMCR_   	temp_tmcr;
	RAMBAR_		temp_trambar;


 	PMMXSync = 0;
	currentSyncStallTooth = 80;

	tooth = 0;
	synch = 0;
	stall = 0;
	shortgap = 0;
	noise = 0;
	syncstall = 0;
	timeout = 0;
	other = 0;
	tpureset = 0;

   
    /* set up TPU Emulation Ram
    */
    temp_trambar.w       = 0x0001;			/* reset value								*/
    temp_trambar.b.ADDR  = 0xFFA0;			/* Must be used on mpc555					*/
    tramreg->trambar.w   = temp_trambar.w;  /* word write to trambar, auto clears RAMDS	*/
    InstallTPUEmuRam();						/* copy microcode into emulation ram		*/
    
	/* set up for 2 microsecond clock on TCR1 for PMMX use, 16 us clock on TCR2 for utility use 		
    */
	temp_tmcr.w          = 0x0080;			/* reset value								*/
	temp_tmcr.b.STOP     = 0x0000;			/* operate clocks normally					*/
	temp_tmcr.b.PRSCL1   = 0x0003;			/* TCR1 prescaler : divide by 8				*/
/*	temp_tmcr.b.PRSCL1   = 0x0002;			/* TCR1 prescaler : divide by 4				*/
	temp_tmcr.b.PRSCL2   = 0x0003;			/* TCR2 prescaler : divide by 8				*/
	temp_tmcr.b.EMU      = 1;				/* emulation switch, 0:normal, 1:use emuram	*/
	temp_tmcr.b.T2CG     = 1;				/* use system clock/8 for input to PRSCL2	*/
	temp_tmcr.b.TPUSUPV  = 0x0001;			/* supervisor mode							*/
/*	temp_tmcr.b.PSCK     = 0x0001;			/* prescaler clock: sys clock/4 to TCR1 	*/
	temp_tmcr.b.PSCK     = 0x0000;			/* prescaler clock: sys clock/32 to TCR1 	*/
	temp_tmcr.b.TPU3     = 0x0001;			/* TPU3 disable, TPU enable for MPC555		*/

/*	Not used on mpc555
	temp_tmcr.b.IARBID   = TPU_IARB;		/* interrupt arbitration value				*/

    tpureg->tmcr.w = temp_tmcr.w;           /* word write to tmcr			            */



/*	Only for MPC555, can change TCR1 prescaler
	 put16(0x30402A,0x005F);	// TPU enhanced pre-scaler control, divide SYSCLOCK by 64
	 put16(0x30402A,0x0043);	// TPU enhanced pre-scaler control, divide SYSCLOCK by 8
*/
	 
	/*	 
	compute clock freq in ticks/ms, note fixed point values
	*/
	
	/*	
	TCR1ifreq = ( SYSCLK / ( temp_tmcr.b.PSCK ? 4 : 32 ) ) >> temp_tmcr.b.PRSCL1;
	
	TCR1freq = Divfx( itofx(( ( (SYSCLK/100) / ( temp_tmcr.b.PSCK ? 4 : 32 ) )
		>> temp_tmcr.b.PRSCL1 ) ), itofx(10) );

	TCR2freq = Divfx( itofx( (SYSCLK/100) / ( 8 * temp_tmcr.b.PRSCL2 ) ), itofx(10) );

	*/


	/* compute clock freq in ticks/ms, note fixed point values
    */
    TCR1freq = Divfx( itofx(10), itofx(( ( (SYSCLK/100) / ( temp_tmcr.b.PSCK ? 4 : 32 ) )
		>> temp_tmcr.b.PRSCL1 ) ) );
    

	/* channel selections
    **
    **
    ** Priorities 01:low, 10:med, 11:high
    **
    **          Use         cfsr        cpr         cier   
    **  ---------------------------------------------------
    **  0       cam in      $B,ITC      10(lo)      0           
    **  1       crank in    $F,PMMX     11(hi)      0
    **  2       spark1      $E,PSP      10(med)     0
    **  3       spark2      $E,PSP      10(med)     0
    **  4       spark3      $E,PSP      10(med)     0
    **  5       spark4      $E,PSP      10(med)     0
    **  6       fuel1       $E,PSP      10(med)     0
    **  7       fuel2       $E,PSP      10(med)     0
    **  8       fuel3       $E,PSP      10(med)     0
    **  9       fuel4       $E,PSP      10(med)     0
    ** 10       unused      --          00          0
    ** 11       iac         $D,SM       01(lo)      0
    ** 12       iac         $D,SM       01(lo)      0
    ** 13       unused      --          00          0
    ** 14       usused      --          00          0
    ** 15       timer       $C,RWTPIN   01(lo)      0
    **  
    */
    
    /* Channel Function Selects
    */
    tpureg->cfsr[0]	= 0xC00D;				
    tpureg->cfsr[1]	= 0xD0EE;				
    tpureg->cfsr[2]	= 0xEEEE;				
    tpureg->cfsr[3] = 0xEEFB;
	
	/* Enable Interrupts
	*/
	tpureg->ticr.w = 0x0100;				/* TPU3 Interrupt Configuration Register	*/
											/* Enables interrupt 1 for TPU				*/
	tpureg->cier   = 0x0003;				/* TPU3 Channel Interrupt Enable Register	*/
											/* Enables interrupt for chan 1 on tpu		*/
    
    tpureg->cpr[0]  = 0x0000;				/* disable all channels						*/
    tpureg->cpr[1]  = 0x0000;				/* disable all channels						*/


	ConfigDPRam();


	/* host sequence registers
    */
    tpureg->hsqr[0]            = 0x0285;
    tpureg->hsqr[1]            = 0x5551;	/* channel 2-7, Time-Angle, time priority,	*/
											/* itc continuous no links					*/

	/* host service request registers
	Initialize all functions
    */

    tpureg->hsrr[0]            = 0x400F; 	/* HSRR 0x01 : read both TCR's 				*/
    tpureg->hsrr[1]            = 0xFFFD;	/* ITC init as well as below */

	/* set channel priorities
	Basically starts the functions here.  All except the spark.  Gets started once synced
    */
    tpureg->cpr[0] = 0x4000;
/*	tpureg->cpr[1] = 0xAAAC;				/* this turns all channels on here			*/
    tpureg->cpr[1] = 0x000D; 
    /* hold until all kicked off
    */
    while( tpureg->hsrr[1] & tpureg->hsrr[0] );
    
}

void ConfigDPRam(void)
{
#define DWELLTIME 0x03E8

	/* channel 0 ITC
    */
	tpudpram->dp[0][TPURAM_W0] = 0x0007;	/* CHAN CONTROL : rising edge, capture and match tcr1	*/
	tpudpram->dp[0][TPURAM_W2] = 0x000E;	/* BANK ADDRESS : points to unimplemented RAM			*/
	tpudpram->dp[0][TPURAM_W4] = 0x0001;	/* MAX COUNT:		1				*/
	tpudpram->dp[0][TPURAM_W6] = 0x0000;	/* TRANS_COUNT 			*/
	tpudpram->dp[0][TPURAM_W8] = 0x0000;	/* FINAL_TRANS_TIME		*/
	tpudpram->dp[0][TPURAM_WA] = 0x0000;	/* LAST_TRANS_TIME		*/ 

	/* channel 1 PMMX
    */
	tpudpram->dp[1][TPURAM_W0] = 0x0004;	/* CONFIG : rising edge, capture and match tcr1						*/
	tpudpram->dp[1][TPURAM_W2] = 0x000B;	/* WAIT TEETH : wait 30 teeth before sync attempt					*/
/*	tpudpram->dp[1][TPURAM_W2] = 0x001E;	/* WAIT TEETH : wait 30 teeth before sync attempt					*/
 
/*	tpudpram->dp[1][TPURAM_W4] = 0x00B4;	/* STATUS:INT_TOOTH, clear status, interrupt on every tooth		*/
	tpudpram->dp[1][TPURAM_W4] = 0x007F;	/* STATUS:INT_TOOTH, clear status, inhibit interrupts				*/
	tpudpram->dp[1][TPURAM_W6] = 0x0514;	/* START PERIOD:NUM PERIODS, tooth 5, 20 tooth long period average */
/*	tpudpram->dp[1][TPURAM_W6] = 0x050A;	/* START PERIOD:NUM PERIODS, tooth 5, 10 tooth long period average */
/*	tpudpram->dp[1][TPURAM_W6] = 0x0505;	/* START PERIOD:NUM PERIODS, tooth 5, 5 tooth long period average */
/*   tpudpram->dp[1][TPURAM_W6] = 0x0F10;	/* START PERIOD:NUM PERIODS, tooth 16, 16 tooth long period average */

/*	tpudpram->dp[1][TPURAM_W8] = 0x4600;    /* RATIO:COUNT_360, 1.2, start count at tooth 0    */
	tpudpram->dp[1][TPURAM_W8] = 0xA000;    /* RATIO:COUNT_360, 2.5, start count at tooth 0    */
/*	tpudpram->dp[1][TPURAM_W8] = 0x8000;    /* RATIO:COUNT_360, 2.0, start count at tooth 0    */
/*	tpudpram->dp[1][TPURAM_W8] = 0x6000;    /* RATIO:COUNT_360, 1.5, start count at tooth 0    */
/*	tpudpram->dp[1][TPURAM_W8] = 0xC000;    /* RATIO:COUNT_360, 3.0, start count at tooth 0    */
/*	tpudpram->dp[1][TPURAM_W8] = 0xE000;    /* RATIO:COUNT_360, 3.5, start count at tooth 0    */
/*	tpudpram->dp[1][TPURAM_W8] = 0x2000;    /* RATIO:COUNT_360, 0.5, start count at tooth 0    */
/*	tpudpram->dp[1][TPURAM_W8] = 0x601E;	/* RATIO:COUNT_360, 1.5, start count at tooth 30  					*/

	tpudpram->dp[1][TPURAM_WA] = 0x2800;	/* LINK_START;LINK_COUNT:COUNT_720, start links at chan 2, 8 chans	*/ 

	/* channel 2 PSP
    */
	tpudpram->dp[2][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[2][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 1000 X 2 microsec close = 2 ms	*/
 	tpudpram->dp[2][TPURAM_W4] = 0x3C00;  	/* END_ANGLE, tooth 60							*/
	tpudpram->dp[2][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[2][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 3 PSP
    */
	tpudpram->dp[3][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[3][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[3][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[3][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[3][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 4 PSP
    */
 	tpudpram->dp[4][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[4][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[4][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[4][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[4][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 5 PSP
    */
 	tpudpram->dp[5][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[5][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[5][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[5][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[5][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 6 PSP
    */
 	tpudpram->dp[6][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[6][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[6][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[6][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[6][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 7 PSP
    */
 	tpudpram->dp[7][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[7][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[7][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[7][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[7][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 8 PSP
    */
 	tpudpram->dp[8][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[8][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[8][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[8][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[8][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 

	/* channel 9 PSP
    */
 	tpudpram->dp[9][TPURAM_W0] = 0x008A; 	/* CONFIG, pin is output, force low, use TCR1	*/
	tpudpram->dp[9][TPURAM_W2] = DWELLTIME;	/* HIGH_TIME, 2000 X 2 microsec close = 4 ms	*/
	tpudpram->dp[9][TPURAM_W4] = 0x1D00;  	/* END_ANGLE, tooth 30							*/
	tpudpram->dp[9][TPURAM_W8] = 0x1D00;  	/* END_ANGLE_ACTIVE = END_ANGLE					*/
	tpudpram->dp[9][TPURAM_WA] = 0x3C18;  	/* SPACING:COUNT_PTR							*/ 
    
	/* channel 11, 12 Stepper motor 
	*/    
	tpudpram->dp[11][TPURAM_W2]  = 0x3333;	/* pin control          						*/
	tpudpram->dp[12][TPURAM_W2]  = 0x9999;	/* pin control          						*/
	tpudpram->dp[11][TPURAM_W0]  = 0x0082;	/* control pin LOW  							*/
	tpudpram->dp[12][TPURAM_W0]  = 0x0081;	/* control pin HIGH 							*/
	tpudpram->dp[11][TPURAM_W4]  = 0x7fff;	/* set current position 						*/
	tpudpram->dp[12][TPURAM_W4]  = 0x0010;	/* set P2 STEP_CNTL_0 							*/
	tpudpram->dp[12][TPURAM_W6]  = 0x05BC;	/* set P3 STEP_CNTL_1 							*/
	tpudpram->dp[11][TPURAM_W8]  = 0x0001;	/* set NEXT_STEP_RATE=1 & MOD_CNT=0 			*/
	tpudpram->dp[11][TPURAM_WA]  = 0x010F;	/* set STEP_RATE_CNT ? (1 to 14) & specify the last secondary channel number=7 */
    								 
	/* channel 15 RWTPIN
    */
	tpudpram->dp[15][TPURAM_W2] = 0x01FF;	/* CHAN_CONTROL : Do not control pin	*/

	TPUsparkadj(itofx(0));

}

void TPUIntrHndlr(void)
{
	unsigned short cisr;			/* TPU Channel Interrupt Status Register		*/
	u_char tpu_errnum;
	unsigned int rpm;

	unsigned int i;


	
	u_short dwell;

	char intrString[50];

/*	cisr = tpureg->cisr;    		/* CISR											*/

	if(GetCISR(1))
	{
		/* ppmx interrupt Channel 1
		*/
		
		tpu_errnum = TPUgetstatus();


#define TOOTH_INTERRUPT		0x00
#define SYNCH 				0x01
#define STALL 				0x81
#define SHORTGAP 			0x82
#define NOISE 				0x83
#define SYNC_STALL 			0x84
#define TIMEOUT 			0x85
		
		switch ( tpu_errnum )
		{
			case TOOTH_INTERRUPT :
				tooth++;
/*				rpm = TPUgetrpm();

				itoa(rpm, intrString);

				puts2("RPM ");
				puts2(intrString);

				htoa(tpustat >> 8, intrString);

				puts2(" TPUSTAT ");
				puts2(intrString);
				
				dwell = tpudpram->dp[2][TPURAM_W2];

				itoa(dwell, intrString);

				puts2("DWELL ");
				puts2(intrString);

				*/
				
				puts2("tooth inter\n");

				break;
			
			case SYNCH :						/* rx first synch, enable spark and fuel funcs	*/
				synch++;
				puts2("Sync\n ");

				PMMXSync = 1;
		
				tpureg->cpr[0] = 0x400A;
				tpureg->cpr[1] = 0xAAAD;
			
				/* hold until all kicked off
				*/
/*				while( tpureg->hsrr[1] & tpureg->hsrr[0] );				*/
				
				break;

			case STALL :
				stall++;
				puts2("Stall\n ");
				break;

			case SHORTGAP :
				shortgap++;
         	puts2("Edge occurred in gap\n ");
				break;

			case NOISE :
				noise++;
				puts2("Spurious noisy edge detected\n ");
				break;

			case SYNC_STALL :
				syncstall++;

				prevSyncStallTooth =  currentSyncStallTooth;

				currentSyncStallTooth = TPUgetCOUNT_360();

				if(prevSyncStallTooth ==  currentSyncStallTooth - 1)
				{
					/* Two consecutive syncstall errors on consecutive teeth.
					   Time to reset tpu
					*/

					tpuReset();
					
					puts2("Expected tooth not present RESET\n ");
				}

				puts2("Expected tooth not present\n ");
/*				tpuReset();		*/
				break;

			case TIMEOUT :
				timeout++;

				if(TPUgetPeriod() > 5000)
				{
					tpuReset();
					puts2("Scheduled match is out of range RESET\n ");
				}

				puts2("Scheduled match is out of range\n ");
				break;

			default :
				other++;
				puts2("other\n ");
				break;
		}
		
	

/*		ClearCISR(1);		/* Reset Interrupt */

	}
	else if(GetCISR(0))
	{
		cmcount++;
		itoa(cmcount,intrString);
		puts2(intrString);
		puts2("!\n");
		
/*		ClearCISR(0);		/* Reset Interrupt */
	}
	else
	{
		puts2("!?\n");
/*		tpureg->cisr = 0x0000; */
	}
	tpureg->cisr = 0x0000;
}


void tpuReset(void)
{

	u_short i;

	PMMXSync = 0;

	currentSyncStallTooth = 80;
	
	tpureset++;



			/* This turns off spark and fuel, and forces the pin low
			*/ 
			tpudpram->dp[2][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[2][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[3][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[3][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[4][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[4][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[5][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[5][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[6][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[6][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[7][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[7][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[8][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[8][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */
			tpudpram->dp[9][TPURAM_W0] = 0x0013;    /* CONFIG, pin is input, force low */
			tpudpram->dp[9][TPURAM_W2] = 0;			/* HIGH_TIME, 0    */

			/* This forces a reset with the above modifications */
			tpureg->hsrr[0] = 0x000F;
			tpureg->hsrr[1] = 0xFFF0;

			/* Wait until hsrr is done */
			while( tpureg->hsrr[1] & tpureg->hsrr[0] );

			/* Now we get on with buisness, lets disable all tpu channels */
		    tpureg->cpr[0]  = 0x0000;		/* disable PMM, fuel and spark channels				*/
		    tpureg->cpr[1]  = 0x0000;		/* leave RWTPIN, SM, ITC running					*/

		    /*tpureg->cpr[0]  = 0x4000;		 disable PMM, fuel and spark channels				*/
		    /*tpureg->cpr[1]  = 0x0001;		 leave RWTPIN, SM, ITC running						*/

         	/*    wait( 25 );	   */ 		/* wait 25 microsec for current cycle to finish		*/ 
			for(i=0;i<32768;i++);

			/* Reconfigure TPU DP RAM parameters */
			ConfigDPRam();


			/* Initialize all functions */

			tpureg->hsrr[0]            = 0x400F; 	/* HSRR 0x01 : read both TCR's 				*/
			tpureg->hsrr[1]            = 0xFFFD;	/* ITC init as well as below */

			/*Set channel priorities
			  Basically starts the functions here.  All except the spark.  Gets started once synced
			*/
			tpureg->cpr[0] = 0x4000;
			tpureg->cpr[1] = 0x000D;		/* ITC low as well as above */
			
			/* hold until all kicked off
			*/
			while( tpureg->hsrr[1] & tpureg->hsrr[0] );


}

short tpuStepperCmd(short int deltaPos)
{
	short currPos = tpudpram->dp[14][TPURAM_W4];
	tpudpram->dp[14][TPURAM_W6]  = currPos + deltaPos;	/* set desired position 		*/
	tpureg->hsrr[0] = 0x3000; 			    			/* set hsrr=3 for step request 	*/
	
	return currPos;
}

short tpuGetStepperPos(void)
{
	return tpudpram->dp[14][TPURAM_W4];
}


u_short TPUgetRpm()
{
//    return (SYSCLK >> 1) / (tpudpram->dp[1][TPURAM_W0]);
//	if(PMMXSync)
		return (SYSCLK / 2 / 32 / 8) / (tpudpram->dp[1][TPURAM_W0] / 20);			//	CONFIGURED FOR 555
//		return (SYSCLK / 4) / (tpudpram->dp[1][TPURAM_W0]);			//	CONFIGURED FOR 555
		//		else return 0;
}

u_short TPUgetPeriod()
{

		return tpudpram->dp[15][TPURAM_WC];			//	CONFIGURED FOR 555

}

u_short TPUgetLongperiod()
{

		return tpudpram->dp[1][TPURAM_W0];	

}

u_short TPUgetCOUNT_360()
{
		return (0x00FF & tpudpram->dp[1][TPURAM_W8]);			//	CONFIGURED FOR 555

}

u_char TPUgetstatus()
{
	u_short t_stat;
    
    t_stat = tpudpram->dp[1][TPURAM_W4];
/*    tpudpram->dp[1][TPURAM_W4] = 0x007D;	/* reset status word		*/
    tpudpram->dp[1][TPURAM_W4] = t_stat & 0x00FF;	/* reset status word		*/

	 t_stat = t_stat >> 8;

    return (u_char)t_stat;
}


unsigned long TPUgetTCR2()
{
    tpureg->hsrr[0] |= 0x4000;   	   		/* set channel 15 hsr=01 to read TCR's		*/ 
    while( tpureg->hsrr[0] & 0xC000 );	 	/* wait til hsr serviced (==0) to get TCR's	*/
 
    return tpudpram->dp[15][TPURAM_WA];    
}

    

void InstallTPUEmuRam()
{
    register int i = sizeof(emuram);
    register char *s, *d;

    d = (char *) EMURAMBASE;
    s = emuram;
    do 
    {
	*d++ = *s++;
    } while (--i);
}	    


void TPUsparkadj(fixed advance)
{
	fixed temp_adv;

	temp_adv = advance / 6;												/*	this is the advance in teeth in fixed*/

	tpudpram->dp[2][TPURAM_W4] = (u_short)(15360 - temp_adv);				/* 15360 is 60 teeth in fixed
																					END_ANGLE, now this is 60 teeth minus advance	*/
	tpudpram->dp[2][TPURAM_W8] = tpudpram->dp[2][TPURAM_W4]; /* END_ANGLE_ACTIVE = END_ANGLE				*/
 
	tpudpram->dp[3][TPURAM_W4] = (u_short)(7680 - temp_adv);  					/* 7680 is 60 teeth in fixed
																					END_ANGLE, now this is 60 teeth minus advance	*/
	tpudpram->dp[3][TPURAM_W8] = tpudpram->dp[2][TPURAM_W4]; /* END_ANGLE_ACTIVE = END_ANGLE				*/
 
      
/*	tm.uparm1 = tpudpram->dp[2][TPURAM_W0];
    tm.uparm2 = tpudpram->dp[2][TPURAM_W2];
    tm.uparm3 = tpudpram->dp[2][TPURAM_W4];
    tm.uparm4 = tpudpram->dp[2][TPURAM_W6];
    tm.uparm5 = tpudpram->dp[2][TPURAM_W8];
    tm.uparm6 = tpudpram->dp[2][TPURAM_WA];
    tm.fparm3 = fxtof(advance);
    tm.fparm4 = fxtof(TCR1freq);*/
}

void TPUdwellup(u_short updown)
{
	u_short dwell_counts;
    
	dwell_counts = tpudpram->dp[2][TPURAM_W2];	/* dwell in counts	*/

	if(updown) dwell_counts++;
	else dwell_counts--;
	
	tpudpram->dp[2][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[3][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[4][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[5][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[6][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[7][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[8][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/
	tpudpram->dp[9][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/

}

void TPUdwelladj(fixed dwell)
{
	long dwell_counts;
    
	dwell_counts = fxtoi( Mulfx( dwell, TCR1freq ) );	/* dwell in ms				*/
	tpudpram->dp[2][TPURAM_W2] = (u_short)dwell_counts;  	/* HIGH_TIME				*/

/*	tm.uparm1 = tpudpram->dp[2][TPURAM_W0];
    tm.uparm2 = tpudpram->dp[2][TPURAM_W2];
    tm.uparm3 = tpudpram->dp[2][TPURAM_W4];
    tm.uparm4 = tpudpram->dp[2][TPURAM_W6];
    tm.uparm5 = tpudpram->dp[2][TPURAM_W8];
    tm.uparm6 = tpudpram->dp[2][TPURAM_WA];
    tm.fparm3 = (float) dwell_counts;
    tm.fparm4 = fxtof( TCR1freq );
    tm.fparm5 = fxtof(dwell);
    tm.fparm9 = fxtof( TCR1freq );
    tm.fparm10= fxtof( Mulfx( dwell, TCR1freq ) );
*/
}

int GetCISR(int channel)
{
	return (tpureg->cisr & (1 << (channel&0xf) ));
}

int ClearCISR(int channel)
{
u_short mask;

	mask = (1 << (channel & 0xf));
	if (tpureg->cisr & mask) {
		tpureg->cisr &= ~mask;
		return 1;
	} else {
		return 0;
	}
}

/*

void SetCFSR(int channel, TPU_FUNC func)
{
	SetBits(&(tpureg->cfsr[3-channel/4]), func, (channel%4)*4, 4);
}

TPU_FUNC GetCFSR(int channel)
{
	return GetBits(&(tpureg->cfsr[3-channel/4]), (channel%4)*4, 4);
}

void SetHSQR(int channel, int hsq)
{
	SetBits(&(tpureg->hsqr[1-channel/8]), hsq, (channel%8)*2, 2);
}

int GetHSQR(int channel)
{
	return GetBits(&(tpureg->hsqr[1-channel/8]), (channel%8)*2, 2);
}

void SetHSRR(int channel, int hsr)
{
	SetBits(&(tpureg->hsrr[1-channel/8]), hsr, (channel%8)*2, 2);
}

int GetHSRR(int channel)
{
	return GetBits(&(tpureg->hsrr[1-channel/8]), (channel%8)*2, 2);
}

void SetCPR(int channel, int cpr)
{
	SetBits(&(tpureg->cpr[1-channel/8]), cpr, (channel%8)*2, 2);
}

void SetCPRs(int which, int prio)
{
u_short cpr[2];
register unsigned int ch = 0;
	cpr[0] = tpureg->cpr[0];
	cpr[1] = tpureg->cpr[1];
	do {
		if (which & 1) {
			SetBits(&cpr[1-(ch / 8)], prio, (ch % 8) *2, 2);
		}
		which >>= 1;
	} while (++ch < 16);
	tpureg->cpr[0] = cpr[0];
	tpureg->cpr[1] = cpr[1];
}

int GetCPR(int channel)
{
	return GetBits(&(tpureg->cpr[1-channel/8]), (channel%8)*2, 2);
}

void SetCIER(int channel, int mit)
{
	SetBits(&(tpureg->cier), mit, channel, 1);
}

int GetCIER(int channel)
{
	return (tpureg->cier & (1 << (channel&0xf) ));
}

int GetCISR(int channel)
{
	return (tpureg->cisr & (1 << (channel&0xf) ));
}

int ClearCISR(int channel)
{
u_short mask;

	mask = (1 << (channel & 0xf));
	if (tpureg->cisr & mask) {
		tpureg->cisr &= ~mask;
		return 1;
	} else {
		return 0;
	}
}

void SetPRAM(int channel, int index, int cont, int lsb, int n)
{
	SetBits(&(tpudpram->dp[channel][index]), cont, lsb, n);
}

int GetPRAM(int channel, int index, int lsb, int n)
{
	return GetBits(&(tpudpram->dp[channel][index]), lsb, n);
}

void SetTBS(int channel, int mit)
{
	SetBits(&(tpudpram->dp[channel][TPURAM_W0]), mit, 5, 4);
}

void SetPAC(int channel, int mit)
{
	SetBits(&(tpudpram->dp[channel][TPURAM_W0]), mit, 2, 3);
}

void SetPSC(int channel, int mit)
{
	SetBits(&(tpudpram->dp[channel][TPURAM_W0]), mit, 0, 2);
}

int GetPSC(int channel)
{
	return GetBits(&(tpudpram->dp[channel][TPURAM_W0]), 0, 2);
}

void SetTBS_PAC_PSC(int channel, int tbs, int pac, int psc)
{
	SetBits(&(tpudpram->dp[channel][TPURAM_W0]), 
			(tbs & 0xf) * 32 + (pac & 0x7) * 4 + (psc & 0x3), 
			0, 9);
}
*/
