/*
	filename here

	Copyright (c) 2000, Peter Siebold

	This file is part of the aems555 project.

	aems555 software is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	aems555 software is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with any aems555 software.  If not, see <http://www.gnu.org/licenses/>.
	
	Many thanks to Peter Siebold for allowing us to redistribute his code under GPL!
*/


/*----------------------------------------------------------------------------------**
** Copyright (c) 2000, Peter Siebold / Jeff Clark                                   **
** All rights reserved.                                                             **
** Redistribution and use in source and binary forms, with or without modification, **
** are permitted provided that the following conditions are met:                    **
**                                                                                  **
**                                                                                  **
** Redistributions of source code must retain the above copyright notice, this list **
**  of conditions and the following disclaimer.                                     **
**                                                                                  **
** Redistributions in binary form must reproduce the above copyright notice, this   **
** list of conditions and the following disclaimer in the documentation and/or      **
** other materials provided with the distribution.                                  **
**                                                                                  **
** Neither the name of "aems555" nor the names of its contributors may be used       **
** to endorse or promote products derived from this software without specific prior **
** written permission.                                                              **
**                                                                                  **
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''    **
** AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE        **
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE   **
** DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY      **
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES       **
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;     **
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON   **
** ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT          **
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS    **
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                     **
**----------------------------------------------------------------------------------**
**																					**
**----------------------------------------------------------------------------------**
**    EFI tpu code for the ppc555													**
**																					**
**	File:		tpu.h																**
**	Desc:		Support for the MPC555 TPU3 using PMMX, PMMA, ITC functions			**
**              This code had its beginnings from Jeff Clark's efi332 tpu code		**
**              but since been completely reworked for MPC555 and Lycoming O-360	**
**																					**
**																					**
**	Author:		Peter Siebold / Jeff Clark											**
**	Tools:   	GNU tool chain														**
**																					**
**----------------------------------------------------------------------------------*/


#ifndef		__tpu555_H
#define		__tpu555_H


typedef union
{
    struct
    {
	    volatile u_short		STOP	:	1;
    	volatile u_short		PRSCL1	:	2;
    	volatile u_short		PRSCL2	:	2;
    	volatile u_short		EMU		:	1;
	    volatile u_short		T2CG	:	1;
    	volatile u_short		STF		:	1;
    	volatile u_short		TPUSUPV	:	1;
    	volatile u_short		PSCK	:	1;
		volatile u_short		TPU3	:	1;
		volatile u_short		T2CSL	:	1;
    	volatile u_short				:	4;
    } b;
    short w;
} TPUMCR_;				//Configured for the 555

typedef union
{
    struct  
    {
    	u_short	:16;
    } b;
    short w;
} TCR_;					//Configured for the 555


typedef union
{
    struct 
    {
    	volatile u_short	HOT4:1;
    	u_short	:4;
    	volatile u_short	BLC:1;
    	volatile u_short	CLKS:1;
    	volatile u_short	FRZ:2;
    	volatile u_short	CCL:1;
    	volatile u_short	BP:1;
    	volatile u_short	BC:1;
    	volatile u_short	BH:1;
    	volatile u_short	BL:1;
    	volatile u_short	BM:1;
    	volatile u_short	BT:1;
    }b;
    short w;
} DSCR_;				//Configured for the 555


typedef union
{
    struct 
    {
	    u_short	:8;
	    volatile u_short	BKPT:1;
	    volatile u_short	PCBK:1;
	    volatile u_short	CHBK:1;
	    volatile u_short	SRBK:1;
	    volatile u_short	TPUF:1;
	    u_short	:3;
    } b;
    short w;
} DSSR_;				//Configured for the 555


typedef union
{
    struct
    {
	    u_short	:5;
	    volatile u_short	CIRL	:3;	/* channel interrupt request level	*/
	    volatile u_short	ILBS	:2;	/* interrupt level byte select		*/
		u_short	:6;
    } b;
    short w;
} TICR_;							//Configured for the 555


typedef struct TPU_ {
	volatile TPUMCR_	tmcr;		//Configured for the 555
	volatile TCR_		ttcr;		//Configured for the 555
	volatile DSCR_      dscr;		//Configured for the 555
	volatile DSSR_      dssr;		//Configured for the 555
	volatile TICR_      ticr;		//Configured for the 555
	volatile u_short	cier;		//Configured for the 555
	volatile u_short	cfsr[4];	//Configured for the 555
	volatile u_short	hsqr[2];	//Configured for the 555
	volatile u_short	hsrr[2];	//Configured for the 555
	volatile u_short	cpr[2];		//Configured for the 555
	volatile u_short	cisr;		//Configured for the 555
} TPUREG;

#define	TPURAM_W0	0
#define	TPURAM_W2	1
#define	TPURAM_W4	2
#define	TPURAM_W6	3
#define	TPURAM_W8	4
#define	TPURAM_WA	5
#define	TPURAM_WC	6
#define	TPURAM_WE	7


typedef struct _tpu_dp_ram {
	volatile u_short dp[16][8];
} TPUDPRAM;


typedef union
{
    struct 
    {
	    volatile u_short	STOP:1;
	    u_short :4;
		volatile u_short    MISF:1;
		volatile u_short    MISEN:1;
	    volatile u_short    RASP:1;
	    u_short :8;
    } b;
    short w;
} DPTMCR_;				//Configured for the 555


typedef union
{
    struct
    {
	    volatile u_short    ADDR:11;	/* addr 8:18 */
	    u_short :4;
	    volatile u_short	RAMDS:1;
    } b;
    short w;
} RAMBAR_;				//Configured for the 555


typedef struct TRAM_ {
	volatile DPTMCR_		trammcr;
	volatile u_short		tramtst;
	volatile RAMBAR_       trambar;
	volatile u_short		tramnotused;
} TRAMREG;				//Configured for the 555



extern void     TPUinit();
extern u_long   TPUgetTCR2();

extern void tpuReset(void);

extern u_short  TPUgetRpm();
extern u_short TPUgetPeriod();
extern u_short TPUgetLongperiod();

extern u_short TPUgetCOUNT_360();

u_char TPUgetstatus();
short tpuGetStepperPos(void);
short tpuStepperCmd(short int deltaPos);
extern void     InstallTPUEmuRam();
extern void     TPUsparkadj(fixed advance);
extern void     TPUdwelladj(fixed delta);
extern void 	TPUdwellup(u_short updown);

typedef enum _TPU_FUNC 
	{SPWM=7, DIO, PWM, ITC, PMM, PSP, SM, OC, PPWA}
TPU_FUNC;

             
extern void     TPUSetPRAM(int channel, int index, int cont, int lsb, int n);
extern int      TPUGetPRAM(int channel, int index, int lsb, int n);
extern void     TPUSetCFSR(int channel, TPU_FUNC func);
extern TPU_FUNC TPUGetCFSR(int channel);
extern void     TPUSetHSQR(int channel, int hsq);
extern int      TPUGetHSQR(int channel);
extern void     TPUSetHSRR(int ch, int hsr);
extern int      TPUGetHSRR(int ch);
extern void     TPUSetCPR(int ch, int cpr);
extern void     TPUSetCPRs(int which, int prio);
extern int      TPUGetCPR(int ch);
extern void     TPUSetCIER(int ch, int mit);
extern int      TPUGetCIER(int ch);
extern int      TPUGetCISR(int ch);
extern int      TPUClearCISR(int ch);
extern void     TPUSetTBS(int ch, int mit);
extern void     TPUSetPAC(int ch, int mit);
extern void     TPUSetPSC(int ch, int mit);
extern int      TPUGetPSC(int ch);
extern void     TPUSetTBS_PAC_PSC(int ch, int tbs, int pac, int psc);


void TPUIntrHndlr(void);
void ConfigDPRam(void);





#endif	/*	__tpu555_H */

